#include <Arduino.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <PacketSerial.h>
#include <LinkedList.h>

#include <Sensor.hpp>

constexpr uint8_t oneWirePort = 4;

enum class PacketType : byte {
	registration = 0x01,
	data = 0x02
};

int main() {
	init();

	PacketSerial packetSerial;
	packetSerial.begin(9600);

	OneWire oneWire(oneWirePort);
	DallasTemperature sensorBus(&oneWire);
	sensorBus.begin();

	LinkedList<Sensor*> sensors;

	DeviceAddress tempAddress;
	Sensor *tempSensorAddress;
	for(int i = 0; i < sensorBus.getDeviceCount(); ++i) {
		sensorBus.getAddress(tempAddress, i);
		tempSensorAddress = new Sensor(i, sensorBus.getResolution(tempAddress), 0.5, sensorBus, tempAddress);
		sensors.add(tempSensorAddress);
	}

	byte buff[Sensor::serializedSize + 1];
	buff[0] = (byte)PacketType::registration;
	for(int i = 0; i < sensors.size(); ++i) {
		sensors.get(i)->registerSensor((byte(&)[Sensor::serializedSize])buff[1]);
		packetSerial.send(buff, sizeof(buff));
	}

	byte dataBuff[Sensor::dataPacketMaxSize + 1];
	dataBuff[0] = (byte)PacketType::data;
	uint16_t written;
	while(true) {
		for(int i = 0; i < sensors.size(); ++i) {
			written = sensors.get(i)->makeDataPacket((byte(&)[Sensor::dataPacketMaxSize])dataBuff[1]);
			packetSerial.send(dataBuff, written);
		}
		delay(500);
	}

	for(int i = 0; i < sensors.size(); ++i) {
		delete sensors.get(i);
	}

	return 0;
}
